//==============================================================================
//         HD44780
//==============================================================================

#ifndef HD44780_H
#define HD44780_H

#include  <stdint.h>
#include  "..\..\system.h"

#define LCD_MODE_4                                        // 4-  
//#define LCD_MODE_8                                        // 8-  

#define PIN_HD44780_EN1     B, 0                          //       
#define PIN_HD44780_EN2     D, 7                          //       
#define PIN_HD44780_RS      B, 1                          //   /  

#define HD44780_DATA_DDR    DDRC                          //    LCD
#define HD44780_DATA_PORT   PORTC                         //    LCD

#define HD44780_INIT                        {                                                                                 \
                                              HD44780_DATA_PORT = 0x00;                                 /*  */   \
                                              HD44780_DATA_DDR  = 0xFF;                                 /*  */   \
                                              PIN_CFG(PIN_HD44780_EN1, PIN_OUT, PIN_CLR);                                     \
                                              PIN_CFG(PIN_HD44780_EN2, PIN_OUT, PIN_CLR);                                     \
                                              PIN_CFG(PIN_HD44780_RS,  PIN_OUT, PIN_CLR);                                     \
                                            }

#define HD44780_EN1_0                       PIN_CLR(PIN_HD44780_EN1)
#define HD44780_EN1_1                       PIN_SET(PIN_HD44780_EN1)
#define HD44780_EN2_0                       PIN_CLR(PIN_HD44780_EN2)
#define HD44780_EN2_1                       PIN_SET(PIN_HD44780_EN2)
#define HD44780_RS_0                        PIN_CLR(PIN_HD44780_RS)
#define HD44780_RS_1                        PIN_SET(PIN_HD44780_RS)

#define HD44780_LOAD1                       {                                                                       \
                                              HD44780_EN1_1;                                                        \
                                              DELAY_NS(125);                                                        \
                                              HD44780_EN1_0;                                                        \
                                            }

#define HD44780_LOAD2                       {                                                                       \
                                              HD44780_EN2_1;                                                        \
                                              DELAY_NS(125);                                                        \
                                              HD44780_EN2_0;                                                        \
                                            }

#ifdef LCD_MODE_4
  #define HD44780_DATA4H(value)             {                                                                       \
                                              HD44780_DATA_PORT &= 0x0F;                      /*  */   \
                                              HD44780_DATA_PORT |= ((value) & 0xF0);          /*  */   \
                                            }

  #define HD44780_DATA4L(value)             {                                                                       \
                                              HD44780_DATA_PORT &= 0x0F;                      /*  */   \
                                              HD44780_DATA_PORT |= (((value) << 4) & 0xF0);   /*  */   \
                                            }

  #define HD44780_WRITE_CFG(value)          {                                                                       \
                                              HD44780_DATA4H(value);                                                \
                                              HD44780_LOAD1;                                                        \
                                              HD44780_LOAD2;                                                        \
                                            }

  #define HD44780_WRITE0(value)             {                                                                       \
                                              HD44780_DATA4H(value);                                                \
                                              HD44780_LOAD1;                                                        \
                                              HD44780_LOAD2;                                                        \
                                              HD44780_DATA4L(value);                                                \
                                              HD44780_LOAD1;                                                        \
                                              HD44780_LOAD2;                                                        \
                                            }

  #define HD44780_WRITE(value1, value2)     {                                                                       \
                                              HD44780_DATA4H(value1);                                               \
                                              HD44780_LOAD1;                                                        \
                                              HD44780_DATA4L(value1);                                               \
                                              HD44780_LOAD1;                                                        \
                                              HD44780_DATA4H(value2);                                               \
                                              HD44780_LOAD2;                                                        \
                                              HD44780_DATA4L(value2);                                               \
                                              HD44780_LOAD2;                                                        \
                                            }
#endif

#ifdef LCD_MODE_8
  #define HD44780_WRITE0(value)             {                                                                       \
                                              HD44780_DATA_PORT = (value);                                          \
                                              HD44780_LOAD1;                                                        \
                                              HD44780_LOAD2;                                                        \
                                            }

  #define HD44780_WRITE_CFG(value)            HD44780_WRITE0(value);

  #define HD44780_WRITE(value1, value2)     {                                                                       \
                                              HD44780_DATA_PORT = (value1);                                         \
                                              HD44780_LOAD1;                                                        \
                                              HD44780_DATA_PORT = (value2);                                         \
                                              HD44780_LOAD2;                                                        \
                                            }

#endif

#define HD44780_WRITE_CMD(value)            {                                                                       \
                                              HD44780_RS_0;                                                         \
                                              HD44780_WRITE0(value);                                                \
                                            }

#define HD44780_WRITE_DATA(value1, value2)  {                                                                       \
                                              HD44780_RS_1;                                                         \
                                              HD44780_WRITE(value1, value2);                                        \
                                            }

void HD44780_Init(void);
void HD44780_LoadSymbol(void);

#endif  //  HD44780_H
