#include <stdint.h>
#include "system.h"
#include "def_prog.h"
#include "device\bell.h"
#include "device\lcd\lcd_hd44780.h"
#include "device\reg_io.h"
#include "module\alg.h"
#include "module\edata.h"
#include "module\menu.h"
#include "menu\menucfg.h"

extern    uint8_t   MenuID;                  //   

volatile  uint8_t   Flags_0;                 //  
volatile  uint8_t   Key;                     //   
volatile  uint8_t   RegIn_1;                 //  1  
volatile  uint8_t   RegIn_2;                 //  2  
volatile  uint8_t   RegOut_1;                //  1  
volatile  uint8_t   RegOut_2;                //  2  
          uint8_t   CountPass;               //   
          uint8_t   Param_1;                 //  1
          uint16_t  Param_2;                 //  2
          int16_t   Param_3;                 //  3
          uint8_t   Param_4[PARAM_4_MAX];    //  4,5,6,7,8,9
          uint8_t   TimeReturn;              //      , 

//==============================================================================
//    EEPROM
//==============================================================================
static void RestoreSetting(void)
{
//  uint8_t param_1 = 0xFF;
//  uint16_t param_2 = 0xFFFF;
//  int16_t param_3 = 0x1000;
//  uint8_t param_4[PARAM_4_MAX] = {0xFF,0xFF,0xFF,0xFF,0xFF,0xFF};
//  
//  EDATA_WRITE(param_1, &param_1);
//  EDATA_WRITE(param_2, &param_2);
//  EDATA_WRITE(param_3, &param_3);
//  EDATA_WRITE(param_4, &param_4);
//  
//  Param_1 = EDATA_RESTORE_UI8(param_1, MIN_PARAM_1, MAX_PARAM_1, DEF_PARAM_1);
//  Param_2 = EDATA_RESTORE_UI16(param_2, MIN_PARAM_2, MAX_PARAM_2, DEF_PARAM_2);
//  Param_3 = EDATA_RESTORE_SI16(param_3, MIN_PARAM_3, MAX_PARAM_3, DEF_PARAM_3);
//  Param_4[0] = EDATA_RESTORE_UI8(param_4[0], MIN_PARAM_4, MAX_PARAM_4, DEF_PARAM_4);
//  Param_4[1] = EDATA_RESTORE_UI8(param_4[1], MIN_PARAM_4, MAX_PARAM_4, DEF_PARAM_4);
//  Param_4[2] = EDATA_RESTORE_UI8(param_4[2], MIN_PARAM_4, MAX_PARAM_4, DEF_PARAM_4);
//  Param_4[3] = EDATA_RESTORE_UI8(param_4[3], MIN_PARAM_4, MAX_PARAM_4, DEF_PARAM_4);
//  Param_4[4] = EDATA_RESTORE_UI8(param_4[4], MIN_PARAM_4, MAX_PARAM_4, DEF_PARAM_4);
//  Param_4[5] = EDATA_RESTORE_UI8(param_4[5], MIN_PARAM_4, MAX_PARAM_4, DEF_PARAM_4);
  
  EDATA_READ(param_1, &Param_1);
  EDATA_READ(param_2, &Param_2);
  EDATA_READ(param_3, &Param_3);
  EDATA_READ(param_4, &Param_4);
}

//==============================================================================
//  
//==============================================================================
static void InitHardware(void)
{
  RegIO_Init();                                   //   /
  Bell_Init();                                    //  
  LCD_Init();                                     //  
  TIMER_1MS_START;                                //    1 
}

//==============================================================================
//  
//==============================================================================
static void InitVariable(void)
{
  RestoreSetting();                               //    EEPROM
  RESET_RECOUNT_1S;                               //   " 1s"
  RESET_RECOUNT_250MS;                            //   " 250ms"
  RESET_KEY_PRESS;                                //   " "
  REG_OE_ON;                                      //      
}

//==============================================================================
//   (Reset)
//==============================================================================
void main(void)
{
  InitHardware();                                           //  
  __enable_interrupt();                                     //  
//  MenuSetMenu(MENU_VERSION);                                //      
//  MenuShow();                                               //    -//-
  InitVariable();                                           //  
  MenuSetMenu(MENU_DEFAULT);                                //   
  WDR_ENABLED;                                              //   

  while(1)                                                  //   
  {
    _WDR();                                                 //   
    if(RECOUNT_250MS)                                       //  250  ,
    {                                                       // 
      Monitoring();                                         // : 
      Control();                                            // : 
      RESET_RECOUNT_250MS;                                  //   " 250ms"
    }

    if(RECOUNT_1S)                                          //  1  ,
    {                                                       // 
//-----       Default -------------------------
      if(MenuID != MENU_DEFAULT)                            //     ,
      {                                                     // 
        if(!TimeReturn) MenuSetMenu(MENU_DEFAULT);          //     ( )
        else            TimeReturn--;                       //   
      }
      RESET_RECOUNT_1S;                                     //   " 1s"
    }
//-----   ---------------------------------------------------
    if(KEY_PRESS)                                           //   ,
    {                                                       // 
      TimeReturn = TIMEOUT_RETURN;                          //       
      MenuKey(Key);                                         //    
      RESET_KEY_PRESS;                                      //    
    }
    MenuShow();
  }
}
