//============================================================================
// 
//    
//============================================================================
#include  "display.h"

static    uint8_t DisplayBuf0[LCD_SIZE];                  //   0
static    uint8_t DisplayBuf1[LCD_SIZE];                  //   1
          uint8_t *Display = DisplayBuf0;                 //       ()
volatile  uint8_t *DisplayOut = DisplayBuf1;              //        LCD
volatile  uint8_t DisplayBlink[LCD_BLINK_BUF];            //     

//============================================================================
//  
//============================================================================
void DisplayUpdate(void)
{
  if(Display == DisplayBuf1)
  {
    Display = DisplayBuf0;
    DisplayOut = DisplayBuf1;
  }
  else
  {
    Display = DisplayBuf1;
    DisplayOut = DisplayBuf0;
  }
}

//============================================================================
//  
//============================================================================
void DisplayClear(void)
{
  uint8_t count = LCD_SIZE;
  uint8_t *disp;
  
  disp = Display;
  while(count--)
    *disp++ = ' ';
}

//============================================================================
//      
//============================================================================
void ShowChar(uint8_t pos, uint8_t ch)
{
  Display[pos] = ch;
}

//============================================================================
//       
//============================================================================
uint8_t ShowText(uint8_t pos, const uint8_t *s)
{
  uint8_t *disp;
  
  disp = Display + pos;
  while(*s)
  {
    *disp++ = *s++;
    pos++;
  }
  return(pos);
}

//============================================================================
//        ( flash)
//============================================================================
uint8_t ShowTextF(uint8_t pos, const uint8_t __flash *s)
{
  uint8_t *disp;
  
  disp = Display + pos;
  while(*s)
  {
    *disp++ = *s++;
    pos++;
  }
  return(pos);
}

//============================================================================
//     8 
//    pos   -  
//    value -  
//    width -   
//    ch    -    
//============================================================================
void ShowValueUI8(uint8_t pos, uint8_t value, uint8_t width, uint8_t ch)
{
  uint8_t *disp;
  uint8_t i;
  
  disp = Display + pos + width;                 //     
  *--disp = value % 10 + '0';
  for(i = 10; width > 1; i *= 10, width--)
  {
    if((value / i) > 0)
      *--disp = (value / i) % 10 + '0';
    else
      *--disp = ch;                             //   ('0'  ' ')
  }
}

//============================================================================
//     16 
//    pos   -  
//    value -  
//    width -   
//    ch    -    
//============================================================================
void ShowValueUI16(uint8_t pos, uint16_t value, uint8_t width, uint8_t ch)
{
  uint8_t *disp;
  uint16_t i;
  
  disp = Display + pos + width;                 //     
  *--disp = value % 10 + '0';
  for(i = 10; width > 1; i *= 10, width--)
  {
    if((value / i) > 0)
      *--disp = (value / i) % 10 + '0';
    else
      *--disp = ch;                             //   ('0'  ' ')
  }
}

//============================================================================
//     8 
//    pos   -  
//    value -  
//    width -   
//    ch    -    
//============================================================================
void ShowValueSI8(uint8_t pos, int8_t value, uint8_t width, uint8_t ch)
{
  uint8_t *disp;
  
  disp = Display + pos;           //     
  if(!(value & 0x80))             //  
    *disp++ = ' ';
  else
  {
    *disp++ = '-';
    value = -value;
  }
  ShowValueUI8(pos+1, value, width, ch);
}

//============================================================================
//     16 
//    pos   -  
//    value -  
//    width -   
//    ch    -    
//============================================================================
void ShowValueSI16(uint8_t pos, int16_t value, uint8_t width, uint8_t ch)
{
  uint8_t *disp;
  
  disp = Display + pos;           //     
  if(value >= 0)           //  
    *disp++ = ' ';
  else
  {
    *disp++ = '-';
    value = -value;
  }
  ShowValueUI16(pos+1, value, width, ch);
}

//============================================================================
//   +  ()
//============================================================================
void ShowTextValueUI8(uint8_t pos, const uint8_t *s, uint8_t value, uint8_t width, uint8_t ch)
{
  pos = ShowText(pos, s);
  ShowValueUI8(pos, value, width, ch);
}

//============================================================================
//   +  ()
//============================================================================
void ShowTextValueUI16(uint8_t pos, const uint8_t *s, uint16_t value, uint8_t width, uint8_t ch)
{
  pos = ShowText(pos, s);
  ShowValueUI16(pos, value, width, ch);
}

//============================================================================
//   +  ()
//============================================================================
void ShowTextValueSI8(uint8_t pos, const uint8_t *s, int8_t value, uint8_t width, uint8_t ch)
{
  pos = ShowText(pos, s);
  ShowValueSI8(pos, value, width, ch);
}

//============================================================================
//   +  ()
//============================================================================
void ShowTextValueSI16(uint8_t pos, const uint8_t *s, int16_t value, uint8_t width, uint8_t ch)
{
  pos = ShowText(pos, s);
  ShowValueSI16(pos, value, width, ch);
}

//============================================================================
//   +  ()
//============================================================================
void ShowTextFValueUI8(uint8_t pos, const uint8_t __flash *s, uint8_t value, uint8_t width, uint8_t ch)
{
  pos = ShowTextF(pos, s);
  ShowValueUI8(pos, value, width, ch);
}

//============================================================================
//   +  ()
//============================================================================
void ShowTextFValueUI16(uint8_t pos, const uint8_t __flash *s, uint16_t value, uint8_t width, uint8_t ch)
{
  pos = ShowTextF(pos, s);
  ShowValueUI16(pos, value, width, ch);
}

//============================================================================
//   +  ()
//============================================================================
void ShowTextFValueSI8(uint8_t pos, const uint8_t __flash *s, int8_t value, uint8_t width, uint8_t ch)
{
  pos = ShowTextF(pos, s);
  ShowValueSI8(pos, value, width, ch);
}

//============================================================================
//   +  ()
//============================================================================
void ShowTextFValueSI16(uint8_t pos, const uint8_t __flash *s, int16_t value, uint8_t width, uint8_t ch)
{
  pos = ShowTextF(pos, s);
  ShowValueSI16(pos, value, width, ch);
}

//==============================================================================
//  
//==============================================================================
void ShowDate(uint8_t pos, uint16_t value)
{
  uint8_t temp;
  uint8_t *disp;
  
  disp = Display + pos;
  temp = BinToBCD(value & 0x001F);
  *disp++ = ((temp >> 4) & 0x0F) + '0';
  *disp++ = (temp & 0x0F) + '0';
  *disp++ = '.';
  value >>= 5;
  temp = BinToBCD(value & 0x000F);
  *disp++ = ((temp >> 4) & 0x0F) + '0';
  *disp++ = (temp & 0x0F) + '0';
  *disp++ = '.';
  *disp++ = '2';
  *disp++ = '0';
  value >>= 4;
  temp = BinToBCD(value & 0x007F);
  *disp++ = ((temp >> 4) & 0x0F) + '0';
  *disp = (temp & 0x0F) + '0';
}

//==============================================================================
//  
//==============================================================================
void ShowTime(uint8_t pos, uint16_t value)
{
  uint8_t temp;
  uint8_t *disp;
  
  disp = Display + pos;
  temp = BinToBCD(value / 60);
  *disp++ = ((temp >> 4) & 0x0F) + '0';   //   
  *disp++ = (temp & 0x0F) + '0';          //   
  *disp++ = ':';
  temp = BinToBCD(value % 60);
  *disp++ = ((temp >> 4) & 0x0F) + '0';   //   
  *disp = (temp & 0x0F) + '0';            //   
}
