#include "edata.h"

//==============================================================================
//    EEPROM
//     :
//      uint32 offset -   EEPROM
//      uint32 count  -    
//      void *dest    -     
// 
//         EDATA_READ
//==============================================================================
void EData_Read(uint32_t offset, uint32_t count, void *dest)
{
  uint8_t *p;
  p = dest;
  
  for(uint32_t i=offset; i<(offset+count); i++)
    *p++ = EEPROM_Read(i);
}

//==============================================================================
//    EEPROM
//     :
//      uint32 offset -   EEPROM
//      uint32 count  -    
//      void *dest    -     
// 
//         EDATA_WRITE
//==============================================================================
void EData_Write(uint32_t offset, uint32_t count, void *dest)
{
  uint8_t *p;
  p = dest;
  
  for(uint32_t i=offset; i<(offset+count); i++)
  {
    if(EEPROM_Read(i) != *p)
      EEPROM_Write(i, *p);
    *p++;
  }
}

//==============================================================================
//    uint8
//     :
//      uint32 offset -  
//      uint8  min    -  
//      uint8  max    -  
//      uint8  def    -   
//     :
//      uint8  value  -  
//==============================================================================
uint8_t EData_RestoreUI8(uint32_t offset, uint8_t min, uint8_t max, uint8_t def)
{
  uint8_t value = EEPROM_Read(offset);
  if((value < min) || (value > max))
  {
    value = def;
    EEPROM_Write(offset, value);
  }
  return(value);
}

//==============================================================================
//    uint16
//     :
//      uint32 offset -  
//      uint16 min    -  
//      uint16 max    -  
//      uint16 def    -   
//     :
//      uint16 value  -  
//==============================================================================
uint16_t EData_RestoreUI16(uint32_t offset, uint16_t min, uint16_t max, uint16_t def)
{
  uint16_t value;

  EData_Read(offset, sizeof(value), &value);
  if((value < min) || (value > max))
  {
    value = def;
    EData_Write(offset, sizeof(value), &value);
  }
  return(value);
}

//==============================================================================
//    int8
//     :
//      uint32 offset -  
//      int8  min    -  
//      int8  max    -  
//      int8  def    -   
//     :
//      int8  value  -  
//==============================================================================
int8_t EData_RestoreSI8(uint32_t offset, int8_t min, int8_t max, int8_t def)
{
  int8_t value = EEPROM_Read(offset);
  if((value < min) || (value > max))
  {
    value = def;
    EEPROM_Write(offset, value);
  }
  return(value);
}

//==============================================================================
//    int16
//     :
//      uint32 offset -  
//      int16 min    -  
//      int16 max    -  
//      int16 def    -   
//     :
//      int16 value  -  
//==============================================================================
int16_t EData_RestoreSI16(uint32_t offset, int16_t min, int16_t max, int16_t def)
{
  int16_t value;
  
  EData_Read(offset, sizeof(value), &value);
  if((value < min) || (value > max))
  {
    value = def;
    EData_Write(offset, sizeof(value), &value);
  }
  return(value);
}
