//==============================================================================
//  
//==============================================================================
#ifndef SYSYEM_H
#define SYSTEM_H

//----- ,   ------------------------------------------
#define FCLK 8000000UL  // Hz
#define TCLK                      (uint64_t)(1000000000UL/(FCLK))
#define DELAY_NS(t)               __delay_cycles((uint64_t)(((t)/(TCLK) != 0) ? ((t)/(TCLK)) : 1))
#define DELAY_US(t)               __delay_cycles((uint64_t)(((((t)*1000UL)/(TCLK)) != 0) ? (((t)*1000UL)/(TCLK)) : 1))
#define DELAY_MS(t)               __delay_cycles((uint64_t)(((((t)*1000000UL)/(TCLK)) != 0) ? (((t)*1000000UL)/(TCLK)) : 1))
#define DELAY_S(t)                __delay_cycles((uint64_t)(((((t)*1000000000UL)/(TCLK)) != 0) ? (((t)*1000000000UL)/(TCLK)) : 1))
#define DELAY_TICK(t)             __delay_cycles((uint64_t)(t))

//-----      -------------------------------------------
#define LOW(x)                    (((uint8_t*)&x)[0])
#define HIGH(x)                   (((uint8_t*)&x)[1])

#define LL(x)                     (((uint8_t*)&x)[0])
#define LH(x)                     (((uint8_t*)&x)[1])
#define HL(x)                     (((uint8_t*)&x)[2])
#define HH(x)                     (((uint8_t*)&x)[3])

//-----      --------------------------------------------
#define BIT_TST(var,bit)          ((var) & (1 << (bit)))
#define BIT_SET(var,bit)          ((var) |= (1 << (bit)))
#define BIT_CLR(var,bit)          ((var) &= (~(1 << (bit))))
#define BIT_INV(var,bit)          ((var) ^= (1 << (bit)))
#define BIT_SET_MASK(var, mask, value)   ((var) = ((var) & ~(mask)) | (value))

//-----    ---------------------------------------------
#define _PIN_IN(port, pin)  DDR##port &= ~(1 << pin)
#define _PIN_OUT(port, pin) DDR##port |= (1 << pin)
#define _PIN_CLR(port, pin) PORT##port &= ~(1 << pin)
#define _PIN_SET(port, pin) PORT##port |= (1 << pin)
#define _PIN_INV(port, pin) PORT##port ^= (1 << pin)
#define _PIN_IDR(port, pin) DDR##port ^= (1 << pin)
#define _PIN_TST(port, pin) (PIN##port & (1 << pin))
#define PIN_IN(port_pin)  _PIN_IN(port_pin)
#define PIN_OUT(port_pin) _PIN_OUT(port_pin)
#define PIN_CLR(port_pin) _PIN_CLR(port_pin)
#define PIN_SET(port_pin) _PIN_SET(port_pin)
#define PIN_INV(port_pin) _PIN_INV(port_pin)
#define PIN_IDR(port_pin) _PIN_IDR(port_pin)
#define PIN_TST(port_pin) _PIN_TST(port_pin)

#define PIN_CFG(port_pin, direct, state)  {                       \
                                            _##state(port_pin);   \
                                            _##direct(port_pin);  \
                                          }

//-----    --------------------------------
#if defined(__ATmega32__)
  #include  <ina90.h>
  #include  <iom32.h>
  #define WDR_INIT                    WDTCR |= (1<<WDTOE) | (1<<WDP2) | (1<<WDP1) | (1<<WDP0)
  #define WDR_ENABLED                 WDTCR |= (1<<WDE)

  #define TIMER_1MS_VECTOR            TIMER0_COMP_vect
  #define TIMER_1MS_INIT              { \
                                          TCNT0 = 0;            /*    */ \
                                          OCR0 = 0x7C;          /*   */ \
                                          BIT_SET_MASK(TCCR0, ((1<<WGM01) | (1<<WGM00)), ((1<<WGM01) | (0<<WGM00)));  /*   CTC */\
                                          TIMSK |= (1<<OCIE0);  /*       */  \
                                      }
  #define TIMER_1MS_START             BIT_SET_MASK(TCCR0, ((1<<CS02) | (1<<CS01) | (1<<CS00)), ((0<<CS02) | (1<<CS01) | (1<<CS00))) //  64

  #define EE_WE                       EEWE
  #define EE_MWE                      EEMWE
  #define EE_RE                       EERE
#endif

#endif  // SYSTEM_H
