#include "menu.h"

TMenuLevel  MenuLevel;                                    //     
uint8_t     MenuSubItem;                                  //    
uint8_t     MenuSubDisplayItem;                           //     
uint8_t     MenuParamVer;                                 //    ()
uint8_t     MenuParamHor;                                 //    ()
uint8_t     EditValueInt8[6];                             //  
uint16_t    EditValueUI16;                                //  
int16_t     EditValueSI16;                                //  

uint8_t MenuID;                                           //   
static uint8_t MenuLastID;                                //   

extern const TMenu __flash *Menu[];                       //   
extern volatile uint8_t Flags_0;                          //  
extern volatile uint8_t DisplayBlink[];                   //     

//==============================================================================
//  
//==============================================================================
void MenuShow(void)
{
  DisplayClear();
  Menu[MenuID]->show();
  DisplayUpdate();
}

//==============================================================================
//   
//==============================================================================
void MenuKey(uint8_t key)
{
  if(key < KEY_COUNT)
    Menu[MenuID]->key[key]();
}

//==============================================================================
//     
//==============================================================================
void MenuSetMenu(uint8_t id)
{
//-----        ---------------------
  if(id != MenuLastID)
  {
    Menu[MenuLastID]->exit();
    MenuLastID = id;
  }
//-----        --------------------
  MenuNoEdit();
  MenuParamHor = 0;
  MenuParamVer = 0;
//-----        --------------------------
  Menu[id]->entry();
  MenuID = id;
}

//==============================================================================
//  
//==============================================================================
void MenuNoEdit(void)
{
  RESET_EDIT;                     //    
  LCD_SET_BLINK(0,0);
}

//==============================================================================
//   
//==============================================================================
void MenuSubShow(uint8_t max, const TSubMenu __flash *s)
{
  uint8_t end = (max > LCD_ROW) ? LCD_ROW : max;
  uint8_t sub_menu = (MenuSubItem == MenuSubDisplayItem) ? 0 : (MenuSubItem - MenuSubDisplayItem);
    
  ShowChar(LCD_POS(MenuSubDisplayItem, 0),'>');
  for(uint8_t i=0; i<end; i++)
    ShowTextF(LCD_POS(i,1), s[sub_menu+i].name);
}

//==============================================================================
//     + 
//==============================================================================
void MenuSubMain(void)
{
  MenuLevel.level = 0;                  // 
  MenuSubItem = 0;                      //    
  MenuSubDisplayItem = 0;               //     
  MenuSetMenu(MENU_SETUP_MAIN);
}

//==============================================================================
//     
//==============================================================================
void MenuSubLevelNext(const TSubMenu __flash *s)
{
  MenuSetMenu(s[MenuSubItem].id_next);
  MenuLevel.menu_item[MenuLevel.level] = MenuSubItem;
  MenuLevel.display_item[MenuLevel.level] = MenuSubDisplayItem;
  MenuLevel.level++;
  MenuSubItem = 0;
  MenuSubDisplayItem = 0;
}

//==============================================================================
//     
//==============================================================================
void MenuSubLevelPrev(uint8_t id)
{
  MenuSetMenu(id);
  MenuLevel.level--;
  MenuSubItem = MenuLevel.menu_item[MenuLevel.level];
  MenuSubDisplayItem = MenuLevel.display_item[MenuLevel.level];
}

//==============================================================================
//     
//==============================================================================
void MenuSubItemNext(uint8_t max)
{
  MenuParamUpUI8(&MenuSubItem, 0, max-1, VAR_END);
  MenuParamUpUI8(&MenuSubDisplayItem, 0, ((max > LCD_ROW) ? LCD_ROW-1 : max-1), VAR_END);
}

//==============================================================================
//     
//==============================================================================
void MenuSubItemPrev(uint8_t max)
{
  MenuParamDownUI8(&MenuSubItem, 0, max-1, VAR_END);
  MenuParamDownUI8(&MenuSubDisplayItem, 0, ((max > LCD_ROW) ? LCD_ROW-1 : max-1), VAR_END);
}

//==============================================================================
//    (, 8 )
//    param -    
//    min   -    
//    max   -    
//    over  -       (0-, 1-)
//==============================================================================
void MenuParamUpUI8(uint8_t *const param, uint8_t min, uint8_t max, uint8_t over)
{
  if(*param < max)  (*param)++;
  else
  {
    if(over) *param = min;
    else     *param = max;
  }
}

//==============================================================================
//    (, 8 )
//    param -    
//    min   -    
//    max   -    
//    over  -       (0-, 1-)
//==============================================================================
void MenuParamDownUI8(uint8_t *const param, uint8_t min, uint8_t max, uint8_t over)
{
  if(*param > min)  (*param)--;
  else
  {
    if(over) *param = max;
    else     *param = min;
  }
}

//==============================================================================
//    (, 16 )
//    param -    
//    min   -    
//    max   -    
//    over  -       (0-, 1-)
//==============================================================================
void MenuParamUpUI16(uint16_t *const param, uint16_t min, uint16_t max, uint8_t over)
{
  if(*param < max)  (*param)++;
  else
  {
    if(over) *param = min;
    else     *param = max;
  }
}

//==============================================================================
//    (, 16 )
//    param -    
//    min   -    
//    max   -    
//    over  -       (0-, 1-)
//==============================================================================
void MenuParamDownUI16(uint16_t *const param, uint16_t min, uint16_t max, uint8_t over)
{
  if(*param > min)  (*param)--;
  else
  {
    if(over) *param = max;
    else     *param = min;
  }
}

//==============================================================================
//    (, 32 )
//    param -    
//    min   -    
//    max   -    
//    over  -       (0-, 1-)
//==============================================================================
void MenuParamUpUI32(uint32_t *const param, uint32_t min, uint32_t max, uint8_t over)
{
  if(*param < max)  (*param)++;
  else
  {
    if(over) *param = min;
    else     *param = max;
  }
}

//==============================================================================
//    (, 32 )
//    param -    
//    min   -    
//    max   -    
//    over  -       (0-, 1-)
//==============================================================================
void MenuParamDownUI32(uint32_t *const param, uint32_t min, uint32_t max, uint8_t over)
{
  if(*param > min)  (*param)--;
  else
  {
    if(over) *param = max;
    else     *param = min;
  }
}

//==============================================================================
//    (, 64 )
//    param -    
//    min   -    
//    max   -    
//    over  -       (0-, 1-)
//==============================================================================
void MenuParamUpUI64(uint64_t *const param, uint64_t min, uint64_t max, uint8_t over)
{
  if(*param < max)  (*param)++;
  else
  {
    if(over) *param = min;
    else     *param = max;
  }
}

//==============================================================================
//    (, 64 )
//    param -    
//    min   -    
//    max   -    
//    over  -       (0-, 1-)
//==============================================================================
void MenuParamDownUI64(uint64_t *const param, uint64_t min, uint64_t max, uint8_t over)
{
  if(*param > min)  (*param)--;
  else
  {
    if(over) *param = max;
    else     *param = min;
  }
}

//==============================================================================
//    (, 8 )
//    param -    
//    min   -    
//    max   -    
//    over  -       (0-, 1-)
//==============================================================================
void MenuParamUpSI8(int8_t *const param, int8_t min, int8_t max, uint8_t over)
{
  if(*param < max)  (*param)++;
  else
  {
    if(over) *param = min;
    else     *param = max;
  }
}

//==============================================================================
//    (, 8 )
//    param -    
//    min   -    
//    max   -    
//    over  -       (0-, 1-)
//==============================================================================
void MenuParamDownSI8(int8_t *const param, int8_t min, int8_t max, uint8_t over)
{
  if(*param > min)  (*param)--;
  else
  {
    if(over) *param = max;
    else     *param = min;
  }
}

//==============================================================================
//    (, 16 )
//    param -    
//    min   -    
//    max   -    
//    over  -       (0-, 1-)
//==============================================================================
void MenuParamUpSI16(int16_t *const param, int16_t min, int16_t max, uint8_t over)
{
  if(*param < max)  (*param)++;
  else
  {
    if(over) *param = min;
    else     *param = max;
  }
}

//==============================================================================
//    (, 16 )
//    param -    
//    min   -    
//    max   -    
//    over  -       (0-, 1-)
//==============================================================================
void MenuParamDownSI16(int16_t *const param, int16_t min, int16_t max, uint8_t over)
{
  if(*param > min)  (*param)--;
  else
  {
    if(over) *param = max;
    else     *param = min;
  }
}

//==============================================================================
//    (, 32 )
//    param -    
//    min   -    
//    max   -    
//    over  -       (0-, 1-)
//==============================================================================
void MenuParamUpSI32(int32_t *const param, int32_t min, int32_t max, uint8_t over)
{
  if(*param < max)  (*param)++;
  else
  {
    if(over) *param = min;
    else     *param = max;
  }
}

//==============================================================================
//    (, 32 )
//    param -    
//    min   -    
//    max   -    
//    over  -       (0-, 1-)
//==============================================================================
void MenuParamDownSI32(int32_t *const param, int32_t min, int32_t max, uint8_t over)
{
  if(*param > min)  (*param)--;
  else
  {
    if(over) *param = max;
    else     *param = min;
  }
}

//==============================================================================
//    (, 64 )
//    param -    
//    min   -    
//    max   -    
//    over  -       (0-, 1-)
//==============================================================================
void MenuParamUpSI64(int64_t *const param, int64_t min, int64_t max, uint8_t over)
{
  if(*param < max)  (*param)++;
  else
  {
    if(over) *param = min;
    else     *param = max;
  }
}

//==============================================================================
//    (, 64 )
//    param -    
//    min   -    
//    max   -    
//    over  -       (0-, 1-)
//==============================================================================
void MenuParamDownSI64(int64_t *const param, int64_t min, int64_t max, uint8_t over)
{
  if(*param > min)  (*param)--;
  else
  {
    if(over) *param = max;
    else     *param = min;
  }
}

//==============================================================================
// 
//==============================================================================
void stub(void)
{
}
